#include <VoltageReference.h>
#include <Adafruit_SSD1306.h>
#include <Fonts/FreeMonoBold12pt7b.h>
#include <SBB_CLICK.h> 
#include <Bounce2.h>   
#include <Adafruit_BMP280.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

Adafruit_BMP280 bmp;

VoltageReference vRef;

#define buzzer 11   

#if (SSD1306_LCDHEIGHT != 32)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif


#define switchpin1 10        // Switch on GPIO10
Bounce btbounce1 = Bounce(); // bounce object for button (is used to get rid of unwanted noise)
SBB_CLICK Button1(10,1);     // Pin 10 1Sek 
boolean longpress = false;   // Global for long press
byte menu = 4;               // Global for Menu // init with last main menu so when the button is pressed menu1 is displayed
byte menusub = 1;            // Global for submenu

boolean beep_on = true;      // Beep on / off
boolean display_on = false;  // Display on / off

// correction of pressure
float pressurecorrecture = 0.0;


// CODE FROM ANDREI's Project https://www.instructables.com/id/DIY-Arduino-Variometer-for-Paragliding
float vario = 0;    
byte samples = 40;    
byte maxsamples = 50;
float alt[51];                                                       
float tim[51];  
float tempo = millis();             // Beschreibung einer Tempovariablen vom Typ float und Zuweisung eines Wertes mit der Funktion millis () - Zählt die Zeit ab Programmstart in Millisekunden
float N1 = 0;                       // Variable zur Mittelwertbildung                                 
float N2 = 0;                       // Variable zur Mittelwertbildung                                
float N3 = 0;                       // Variable zur Mittelwertbildung                                 
float D1 = 0;                       // Variable zur Mittelwertbildung                               
float D2 = 0;                       // Variable zur Mittelwertbildung       

//BEEP
float beep;                                  
float Beep_period;                

// END CODE FROM ANDREI's Project https://www.instructables.com/id/DIY-Arduino-Variometer-for-Paragliding


void setup()   {                
  Serial.begin(9600);

  //Voltage Measurement
  vRef.begin();

  
  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)

  // init the BMP280
  bmp.begin(); 

  // Buzzer output on Pin 11
  pinMode(buzzer, OUTPUT);
  
  // init done
  

  // Clear the buffer.
  display.clearDisplay();


  // wait until the Sensor is ready
  initScreen();


  // Button initialisation
  pinMode(switchpin1, INPUT);       // button auf Eingang setzen
  digitalWrite(switchpin1, HIGH);   // Pullup einschalten fuer button
  btbounce1.attach(switchpin1);     // attach switch to bounce object
  btbounce1.interval(5);

}


void loop() {


    float Altitude    = bmp.readAltitude(1013.25 + pressurecorrecture);     //here you can give an correcture argument     bmp.readAltitude(1013.25 -+ correct); 


    //Steigen berechnen
    VarioMSCalculation();

    // Beep (Treshold up/Treshold Down
    if (beep_on){             // only when beep is on
      VarioBeep(0.2,-5.0);
    }
    
    // Button Handling
    ButtonFunctions();

    
   // Screen Output
   if (display_on){
    if (menu == 1) {
      M1_displayVarioScreen(Altitude,vario);
    }
    else if (menu == 2){
      M2_DisplayClimbBar(vario);
    }
    else if (menu == 3){
      M3_DisplayTempPressure(bmp.readTemperature(), bmp.readPressure()/100.0);
    }
    else if (menu == 4){
      M4_DisplayBattery();
    }
    else if (menu == 5){  
      M5_DisplayPressureSetup(Altitude);
    }
    else if (menu == 6){
      M6_DisplayBeepSetup();
    }
    else if (menu == 7){
      M7_DisplayDisplay_OnOff();      
    }
    else if (menu == 8){
      Mx_DisplayExit();
    }

   // because the display calculation takes some time we have to do also some delay here when the display is off
   }
   else
   {
    delay(39);  // empierisch getestet
   }

   //Serial.println(tempo,0);

}


// When Button is pressed
void ButtonFunctions(){

  // Button short press
  if ((btbounce1.update() == 1) && (btbounce1.read() == 0) && (btbounce1.fell() == 1) && (longpress == false)) {

    // turn back on the display on every short press
    display_on = true;

    // first stage 4 main menus (1. Vario / 2. Vario Bar / 3. TempPress / 4. Battery )
    if (menusub == 1) {
      if (menu == 4){
        menu = 1;
      }
      else {
        menu = menu + 1;  
      }
    }
    // second stage 4 main menus (pressure / beep / display / exit)
    else if (menusub == 2){
      if (menu == 8){
        menu = 5;
      }
      else {
        menu = menu + 1;  
      }
    }
    // aetting stage
    else if (menusub == 3){
      if (menu == 5){    // set pressure
        if (pressurecorrecture >= 8.0){
          pressurecorrecture = -8.0;
        }
        else {
        pressurecorrecture = pressurecorrecture + 0.5;
        }        
      }
      
      else if (menu == 6){
        if (beep_on){
          beep_on = false;
        }
        else {
          beep_on = true;
        }
      }
      
      else if (menu == 7){
        if (display_on){
          display_on = false;
          display.clearDisplay();
          display.display();
          // exit immediatly
          menu = 4;  // last menu so next button press is vario screen
          menusub = 1;
        }
        else {
          display_on = true;
        }
      }

      
    }

    
    Serial.print(F("short press Menu: ")); 
    Serial.println(menu); 
    
  }


  // Button Long press
  if (Button1.click()){

    if (menu <= 4) {
      menusub = 2;
      menu    = 5;
    }
    else if (menu >= 5 && menu <= 7){
      if (menusub == 2){
        menusub = 3;
      }
      else{
        // EXIT
        menu = 1;
        menusub = 1;
      }
      
    }
    else if (menu == 8){
      // EXIT
      menu = 1;
      menusub = 1;
    }
    
    
    Serial.println(F("Long Press 1 sek"));
    longpress = true;  // set longpress
  }


  if (btbounce1.fell() == 1){
    longpress = false;
  }
  Button1.release();

}



// Variofunktion FROM ANDREI's Project https://www.instructables.com/id/DIY-Arduino-Variometer-for-Paragliding
void VarioMSCalculation(){
  tempo = millis();                                                       // Beschreibung einer Tempovariablen vom Typ float und Zuweisung eines Wertes mit der Funktion millis () - Zählt die Zeit ab Programmstart in Millisekunden
  N1 = 0;                                                                 // Variable zur Mittelwertbildung
  N2 = 0;                                                                 // Variable zur Mittelwertbildung
  N3 = 0;                                                                 // Variable zur Mittelwertbildung
  D1 = 0;                                                                 // Variable zur Mittelwertbildung
  D2 = 0;                                                                 // Variable zur Mittelwertbildung
  
  ///// ОБНУЛЕНИЕ ВАРИОМЕТРА / ZERO VARIO /////
  vario = 0;       
  
  
  for(int cc=1; cc<=maxsamples; cc++)                                     // Averager 
  {                                                                       // 
    alt[(cc-1)] = alt[cc];                                                // http://www.instructables.com/id/GoFly-paraglidinghanglidinggliding-altimeter-v/?ALLSTEPS
    tim[(cc-1)] = tim[cc];                                                // http://redhats.ru/variometer-arduino-2015/
  }                                                                       //
                                                                          //
  alt[maxsamples] = bmp.readAltitude();                                   //
  tim[maxsamples] = tempo;                                                //
  float stime = tim[maxsamples-samples];                                  //
                                                                          //
  for(int cc=(maxsamples-samples); cc<maxsamples; cc++)                   //
  {                                                                       //
    N1+=(tim[cc]-stime)*alt[cc];                                          //
    N2+=(tim[cc]-stime);                                                  //
    N3+=(alt[cc]);                                                        //
    D1+=(tim[cc]-stime)*(tim[cc]-stime);                                  //
    D2+=(tim[cc]-stime);                                                  //
  }                                                                       // Durchschnittliches Körperende 
  
  /////VARIO VALUES CALCULATING /////
  vario=1000*((samples*N1)-N2*N3)/(samples*D1-D2*D2);                     // BERECHNUNG VON VARIOMETERWERTEN

  //Serial.print("Vario Russ: ");
  //Serial.println(vario);
}



void VarioBeep(float TreshUp, float TreshDown){

    if ((tempo - beep) > Beep_period + 100)                         
  {                                                                
    beep=tempo;                                                    
    if (vario > TreshUp && vario<15)                                
    {                                                              
      
      Beep_period = 400-(vario*80);       
      
      tone( buzzer , (500 + (250 * vario)), abs(Beep_period));
    }                                             
    else if (vario < TreshDown)                      // Sinking Tone
    {                                              
      Beep_period=200;                                           
      tone(buzzer,200,400);                                    
    }                                                          
  }           
  
}



void initScreen() {
 display.setRotation(0); 
 display.setFont(&FreeMonoBold12pt7b);
 display.setTextColor(WHITE);
 display.setCursor(2,24);
 display.clearDisplay();
 display.print(F("Asynchron"));
// display.drawBitmap(0, 0,  testlogo, 128, 32, 1);

 display.display();

 delay(2000);

 display.clearDisplay(); 
 display.setCursor(2,24);
 display.print(F("  , )( `  "));
 display.display();

 delay(2000);
 
 display.clearDisplay();
 display.display();
}


void draw_battery_symbol(int x, int y, int width, int height, int offset ){
   // Draw Battery
  int vcc = vRef.readVcc();


  // 4200 mv 3500 mv are the min max values of milivolt
  // 10 are the maximum pixels of the filled rect
  // so (((4200.0 - 3500.0)-(4200.0-vcc)) / (4200.0 - 3500.0)) * 10.0 is calculating the pixelsize of the rect (make sure the calculation is in floating value  
  int pixelwidth = (((4200.0 - 3500.0)-(4200.0-vcc)) / (4200.0 - 3500.0)) * (width - 2 - (offset * 2));
  pixelwidth = constrain(pixelwidth,0,width-2-(offset * 2));

  // battery frame: Startpt (X116 Y28, size 12x4
  if (offset == 0) {
    display.drawRect(x, y, width, height, WHITE); 
    display.fillRect(x+1+offset, y + 1 + offset, pixelwidth  , height - 2 - (offset * 2), WHITE);
  }
  else
  {
    display.drawRoundRect(x, y, width, height, offset, WHITE);
    display.fillRoundRect(x+1+offset, y + 1 + offset, pixelwidth  , height - 2 - (offset * 2), offset/2, WHITE);
  }

}


// does the Main Drawing of the Vario Screen
void M1_displayVarioScreen(float Altitude, float Steigen) {

  // avoid Steigen to be out of range (happens always after Initialisation and looks fckd up on the display
  Steigen = constrain(Steigen, -99, 99);
  
  //display.setRotation(0);
  //display.setTextSize(18);
  display.setFont(&FreeMonoBold12pt7b);
  display.setTextColor(WHITE);
  display.clearDisplay();

  // STeiganzeiger bar
  display.drawRect(54, 0, 8, 32, WHITE); 
  if (Steigen > 0) {                                        // STEIGEN
    //display.print("+");
    display.fillRect(54, constrain( 32- Steigen * 10 ,0,32), 8, 32, WHITE);
    // Draw plus 5x5px
    display.drawLine(46, 4, 50, 4, WHITE);
    display.drawLine(48, 2, 48, 6, WHITE);
  }
  else {                                                    // SINKEN
    display.fillRect(54, 0, 8,constrain( abs(Steigen) * 10,0,32), WHITE);
    // Draw minus 1x5px
    display.drawLine(46, 28, 50, 28, WHITE);
  }



  // Steig/Sinkwerte ohne Vorzeichen
  // Shift Cursor wenn Steigen groesser als 10 ms ist
  if (Steigen > 10){
    display.setCursor(14,22);
    display.print(abs(Steigen),0);
  }
  else{
    display.setCursor(0,22);
    display.print(abs(Steigen),1);
  }



  // Shift Altitude to the Right
  if (Altitude < 10){
    display.setCursor(114,22);
  }
  else if (Altitude < 100){
    display.setCursor(98,22);
  }
  else if (Altitude < 1000){
    display.setCursor(82,22);
  }
  else {
    display.setCursor(66,22);
  } 
  display.print(Altitude,0);
  //display.drawRect(0, 0, 62, 32, WHITE);


  draw_battery_symbol(116,28,12,4, 0);


  display.display();

  delay(10);
  //display.setRotation(0);
}


void M2_DisplayClimbBar(float Steigen){

  // avoid Steigen to be out of range (happens always after Initialisation and looks fckd up on the display
  Steigen = constrain(Steigen, -99, 99);
  display.clearDisplay();

  // draw scale
  byte i;

  for (i = 0; i < 128; ++i)
  {
    if (i % 5 == 0){
      if (i % 10 == 0){                          // Big line
        display.drawLine(i, 31, i, 28, WHITE);  
        display.drawLine(i,  0, i, 3, WHITE);
      }
      else{                                      // Small Line
        display.drawLine(i, 31, i, 30, WHITE); 
        display.drawLine(i,  0, i,  1, WHITE); 
      }
      
    }
  }

  //Draw Vertikal line to split climbing and sinking
  display.drawLine(40,  5, 40,  26, WHITE); 

  
  if (Steigen > 0) {
    display.fillRect(41,  5, constrain( Steigen * 10 ,1,88), 22, WHITE);
  }
  else if (Steigen < 0){
    display.fillRect( constrain( 40 - abs(Steigen) * 10 ,0,39),  5,  constrain(abs(Steigen) * 10 ,1,40), 22, WHITE);
  }

  display.display();
  delay(10);
}


// Displays Temperature (C) and Pressure (hPa)
void M3_DisplayTempPressure(float Temperature, float Pressure){

  display.clearDisplay();

  display.setCursor(5,22);
  display.print(Temperature,0);
  display.print(F(" "));
  display.print(Pressure,1);
  
  display.display();
  
  delay(10);
}


void M4_DisplayBattery(){
  int vcc = vRef.readVcc();
 
  display.clearDisplay();

  draw_battery_symbol(5,6,50,20,4);

  display.setCursor(60,22);
  display.print(100.0/(4200.0-3500.0)*(constrain(vcc-3500,0,4200-3500)),0);
  display.print(F("%"));
  display.display();
  
  delay(10);
  
}


// Display Pressure Settup
void M5_DisplayPressureSetup(float Altitude){
  display.clearDisplay();

  display.setCursor(4,22);

  display.print(F("Alt:"));
  display.print(Altitude,0);

  // in edit stage
  if (menusub == 3){
    display.drawRoundRect(0, 0, 128, 32, 3, WHITE);
  }
  else {
    //Settingsmenumarker on right side
    // first submenu
    display.fillRect(127,0,2,32,WHITE);
    display.fillRect(120,0,8,8,WHITE);
  }
   
  display.display();
  delay(10);

}


// Display Pressure Settup
void M6_DisplayBeepSetup(){
  display.clearDisplay();

  display.setCursor(4,22);

  
  if (beep_on){
    display.print(F("Beep: ON"));
  }
  else {
    display.print(F("Beep:OFF"));
  }

  // in edit stage
  if (menusub == 3){
    display.drawRoundRect(0, 0, 128, 32, 3, WHITE);
  }
  else {
    //Settingsmenumarker on right side
    // second submenu
    display.fillRect(127,0,2,32,WHITE);
    display.fillRect(120,8,8,8,WHITE);
  }
  
  display.display();
  delay(10);

}


// Display Pressure Settup
void M7_DisplayDisplay_OnOff(){
  display.clearDisplay();

  display.setCursor(4,22);

  
  if (display_on){
    display.print(F("Disp: ON"));
  }
  else {
    display.print(F("Disp:OFF"));
  }

  // in edit stage
  if (menusub == 3){
    display.drawRoundRect(0, 0, 128, 32, 3, WHITE);
  }
  else {
    //Settingsmenumarker on right side
    // third submenu
    display.fillRect(127,0,2,32,WHITE);
    display.fillRect(120,16,8,8,WHITE);
  }
  
  display.display();
  delay(10);

}



void Mx_DisplayExit(){
  display.clearDisplay();

  display.setCursor(4,22);

  display.print(F("  EXIT"));

  display.drawRoundRect(0, 0, 128, 32, 3, WHITE);
   
  display.display();
  delay(10);
}

